@@ -1,6 +1,6 @@
 module CodeRay
 module Encoders
-
+  
   # = Debug Encoder
   #
   # Fast encoder producing simple debug output.
@@ -10,40 +10,52 @@ module Encoders
   # You cannot fully restore the tokens information from the
   # output, because consecutive :space tokens are merged.
   # Use Tokens#dump for caching purposes.
+  # 
+  # See also: Scanners::Debug
   class Debug < Encoder
-
-    include Streamable
+    
     register_for :debug
-
+    
     FILE_EXTENSION = 'raydebug'
-
-  protected
+    
+    def initialize options = {}
+      super
+      @opened = []
+    end
+    
     def text_token text, kind
       if kind == :space
-        text
+        @out << text
       else
+        # TODO: Escape (
         text = text.gsub(/[)\\]/, '\\\\\0')  # escape ) and \
-        "#{kind}(#{text})"
+        @out << kind.to_s << '(' << text << ')'
       end
     end
-
-    def open_token kind
-      "#{kind}<"
+    
+    def begin_group kind
+      @opened << kind
+      @out << kind.to_s << '<'
     end
-
-    def close_token kind
-      ">"
+    
+    def end_group kind
+      if @opened.last != kind
+        puts @out
+        raise "we are inside #{@opened.inspect}, not #{kind}"
+      end
+      @opened.pop
+      @out << '>'
     end
-
+    
     def begin_line kind
-      "#{kind}["
+      @out << kind.to_s << '['
     end
-
+    
     def end_line kind
-      "]"
+      @out << ']'
     end
-
+    
   end
-
+  
 end
 end
